/*
 *
 * Perforce/JBuilder Opentool
 * Copyright (C) 2002 Mark Ackerman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 package com.dafreels.opentools.actions.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.*;

import com.dafreels.opentools.actions.*;
import com.dafreels.opentools.*;
import com.dafreels.vcs.command.*;
/**
 * Title: Perforce Open Tool
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author Mark Ackerman
 * @version 1.0
 */

public class HistoryDialog extends JDialog
{
	private static final String LEAD_IN      = "... ";
	protected static Vector _columnNames = new Vector(4);

	private JTable _historyTable;
	private JTextArea _fileName;
	static
	{
		_columnNames.add( "Change List" );
		_columnNames.add( "Date" );
		_columnNames.add( "User" );
		_columnNames.add( "Description");
	}

	public HistoryDialog(Frame frame, String title, boolean modal)
	{
		super(frame, title, modal);
		buildControls();
	}
	protected void buildControls()
	{
		getContentPane().setLayout(new GridBagLayout());

		Color gray = new Color(200,200,200);
		JLabel label = new JLabel("File:");
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(5,5,0,5);
		getContentPane().add(label, gbc);

		_fileName = new JTextArea();
		_fileName.setEditable(false);
		_fileName.setBackground(null);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(5,5,5,5);
		getContentPane().add(_fileName, gbc);


		_historyTable = new JTable()
		{
			public boolean isCellEditable(int row, int col)
			{
				return false;
			}
		};
		_historyTable.setBackground(null);
		//_historyTable.setEnabled(false);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(5,5,5,5);
		getContentPane().add(new JScrollPane(_historyTable), gbc);

		JPanel panel = new JPanel(new GridBagLayout());
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridwidth = 2;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(5,5,5,5);
		getContentPane().add(panel, gbc);

		JButton button = new JButton("OK");
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(5,5,5,5);
		panel.add(button, gbc);

		_historyTable.addMouseListener(new MouseAdapter()
		{
			public void mouseClicked(MouseEvent e)
			{
				if ( e.getClickCount() >= 2 && SwingUtilities.isLeftMouseButton(e))
				{
					int row = _historyTable.rowAtPoint(e.getPoint());
					if ( row == -1 )
					{
						return;
					}
					Vector changeList = getChangeList();
					try
					{
						setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
						new DescribeAction((String)_historyTable.getValueAt(row, 0), changeList);
					}
					finally
					{
						setCursor(Cursor.getDefaultCursor());
					}
				}
			}
		});
		button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				setVisible(false);
			}
		});

	}

	/**
	 * fill this dialog with the contents of formatter
	 * @param formatter the output from the <code>fstat</code> command
	 */
	public void fillForm(String fileName, MessageFormatter formatter)
	{
		if ( _fileName != null )
		{
			_fileName.setText(fileName);
		}
		String message, arg;
		StringTokenizer tokenizer;
		String token;
		int field;
		Vector row;
		Vector table = new Vector();
		MessageWriter.outputErrorMessages(MessageFormatter.getInstance());
		boolean firstFile = true;
		while( (message = formatter.getNextMessage()) != null)
		{
System.out.println(message);
			if ( message.startsWith("//"))
			{
				if ( !firstFile )
				{
					HistoryDialog dialog = new HistoryDialog((Frame)getParent(),
						getTitle(), false);
					dialog.fillForm(message, formatter);
					dialog.setVisible(true);
					break;
				}
				firstFile = false;
			}
			else if ( message.startsWith(LEAD_IN))
			{
				message = message.substring(4);
				if ( message.startsWith(LEAD_IN))
				{
					continue; // proly got a ... ... branch into
				}
// format of message s/b:
// #7 change 60 edit on 2002/04/08 by mark@Q (text) 'fix so that during a Save'
				tokenizer = new StringTokenizer(message, " ");
				field = 0;
				row = new Vector(4);
				while (tokenizer.hasMoreTokens() && field <= 10)
				{
					token = tokenizer.nextToken();
					field++;
					switch ( field )
					{
						case 1: // record number - skip
						case 2: // change type - skip
						case 4: // type of change, edit, add, etc - skip
						case 5: // word 'on' - skip
						case 7: // word 'by'
						case 9: // (text)
						break;
						case 3: // changelist number
							row.add(token);
						break;
						case 6: // date
							row.add(token);
						break;
						case 8: // changed by
							row.add(token);
						break;
						case 10: // description
							int idx = message.indexOf("'");
							if ( idx == -1 )
							{
								continue;
							}
							row.add(message.substring(idx, message.length()-1));
						break;
					}
				}
				table.add(row);
			}
		}
		DefaultTableModel model = new DefaultTableModel(table, _columnNames)
		{
			public boolean isCellEditable(int row, int column)
			{
				return false;
			}
		};
		_historyTable.setModel(model);
		pack();
		setLocation();
	}
	/**
	 * set the dialogs location centered on its parent. If it doesn't
	 * hava a parent then center it on the screen.
	 */
	public void setLocation()
	{
		Component parent = getParent();
		int ix, iy, bx, by;
		Dimension d = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		Dimension dd = getSize();
		if ( parent == null )
		{
			ix = (d.width-dd.width)/2;
			iy = (d.height-dd.height)/2;
		}
		else
		{
			Rectangle bounds = parent.getBounds();
			Rectangle abounds = getBounds();
			ix = bounds.x + (bounds.width - abounds.width)/ 2;
			iy = bounds.y + (bounds.height - abounds.height)/2;
		}
		if(iy < 10) iy = 10;
		if(ix < 10) ix = 10;
		bx = ix+dd.width;
		by = iy+dd.height;
		// see if the dialog is off the screen.
		if ( bx > d.width )
		{
			ix -=bx-d.width;
		}
		if ( by > d.height )
		{
			iy -= by-d.height;
		}
		setLocation(ix,iy);
	}

	protected Vector getChangeList()
	{
		int rowCnt = _historyTable.getRowCount();
		Vector v = new Vector(rowCnt);
		for (int i = 0; i < rowCnt; i++ )
		{
			v.add(_historyTable.getValueAt(i, 0));
		}
		return v;
	}
	public static void main(String[] args)
	{
		new HistoryDialog(new JFrame(), "File History", false).setVisible(true);
	}

}